<?php
class Acces_Controller extends TinyMVC_Controller
{
  
  /*************************
   * Subsistema d'accessos *
   *************************/
	
  /*********************
   * Gestió d'accessos *
   *********************/

  /*
   * Funció que indica si l'usuari administrador està autenticat o no. 
   * Aquesta funció s'utilitza internament per validar si es pot executar o no
   * una determinada acció.
   */
  private function sessioValida() {
    session_start();
  	if (!isset($_SESSION['autenticat']) or $_SESSION['autenticat']==0) {
		header('location:../ram/disconnect');
  	}
  }
  
  /*
   * Funció que s'utilitza per actualitzar el log intern de l'aplicació
   */ 
  private function registrar($descripcio) {
  	$username=$_SESSION['username'];
  	$this->load->model('Logs_Model','logs',null,'default');
  	$this->logs->registrar($username,$descripcio);
  }

  /*
   * Funció que mostra la vista d'alta d'accés, prèvia recol·lecció
   * de les dades necessàries per la vista  
   */
  function altaAccesVista() {
  	$this->sessioValida();
  	// missatge inicial alta accés
  	$missatge['tipus']="msg-ok";
  	$missatge['text']="Donar d'alta un acc&eacute;s al sistema. Si no s'especifica data d'expiraci&oacute;, l'acc&eacute;s<br> no caducar&agrave;.";
  	$this->view->assign('missatge',$missatge);
  
  	// s'obtenen els grups d'usuaris donats d'alta
  	$this->load->model('Usuaris_Model','accessos',null,'default');
  	$ugrups=$this->accessos->consultar_grups();
  	$this->view->assign('ugrups',$ugrups);
  	
  	// s'obtenen els grups de servidors donats d'alta
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	$sgrups=$this->servidors->consultar_grups();
  	$this->view->assign('sgrups',$sgrups);
  	
    $this->view->display('acces_alta_view');
  }

  /*
   * Funció que dóna d'alta un accés a l'aplicació
   */
  function altaAcces() {
	$this->sessioValida();
  	
	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	$this->load->model('Accessos_Model','accessos',null,'default');

  	if (!isset($_POST['ugrups']) or !isset($_POST['sgrups'])) {

		$missatge['tipus']="msg-warn";
  		$missatge['text']="Acc&eacute;s incorrecte: com a m&iacute;nim s'ha de seleccionar un <i>usuari</i> i un <i>servidor</i>";
  		// s'actualitza el missatge amb el resultat de l'insert a la vista corresponent
  		$this->view->assign('missatge',$missatge);
  		
  		$ugrups=$this->usuaris->consultar_grups();
  		$this->view->assign('ugrups',$ugrups);
  		
  		$sgrups=$this->servidors->consultar_grups();
  		$this->view->assign('sgrups',$sgrups);
  		
  		$this->view->display('acces_alta_view');
		
  	} else {
  		
  		$ugrups = $_POST['ugrups'];
  		$sgrups = $_POST['sgrups'];
		
  		$dataexp = date('d-m-Y',strtotime($_POST['dataexp']));
  		$datanull = date('d-m-Y',strtotime('01-01-1970'));
  		
  		if ($dataexp == $datanull) {
  			$dataexp = null;
  		} 		
  		
		for ($i=0; $i<count($ugrups);$i++) {
	
			for ($j=0; $j<count($sgrups);$j++) {
		  		$this->accessos->insertar_acces($ugrups[$i],$sgrups[$j],$dataexp);
				$this->registrar('Alta acces grup usuari ['.$ugrups[$i].'] a grup servidor ['.$sgrups[$j].']');
			}  	
		} 

		// missatge alta ok
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Acc&eacute;s entre donat d'alta correctament.";
  		// s'actualitza el missatge amb el resultat de l'insert a la vista corresponent
  		$this->view->assign('missatge',$missatge);
  		
  		$ugrups=$this->usuaris->consultar_grups();
  		$this->view->assign('ugrups',$ugrups);
  		
  		$sgrups=$this->servidors->consultar_grups();
  		$this->view->assign('sgrups',$sgrups);
  		
  		$this->view->display('acces_alta_view');
  	}
  }  
  
  /* 
   * Funció que mostra tots els accessos donats d'alta a l'aplicació
   */
  function consultaAccessos() {
  	$this->sessioValida();
  	// cas 1: estem davant d'una cerca genèrica, que és la vista inicial
  	// que es presenta o bé l'usuari ha forçat aquesta vista.
  	if (!isset($_POST['cerca']) or $_POST['cerca']=="") {
 
		$this->load->model('Accessos_Model','accessos',null,'default');
  		$accessos = $this->accessos->consultar_accessos();
		$this->view->assign('accessos',$accessos);
		
		// missatge inicial consulta usuaris
  		$missatge['tipus']="msg-ok";
	  	$missatge['text']="Consulta d'accessos. Es pot filtrar la sortida amb l'opci&oacute; <i>cercar</i>.";
		$this->view->assign('missatge',$missatge);
		
		$this->view->display('accessos_consulta_view');

	// cas 2: l'usuari ha escollit fer una cerca, de forma que s'ha de filtrar
	// la sortida que es mostrarà a la vista corresponent
  	} else {
  		
  		// es recuperen els paràmetres de la cerca passats per POST
  		$valor=$_POST['cerca'];
  		$camp=$_POST['opcio'];
  		
  		$this->load->model('Accessos_Model','accessos',null,'default');

  		// en cas que no s'hagi trobat cap usuari que compleixi la cerca, 
  		// no es retorna cap valor (=null) a la variable usuaris de la vista
  		$accessos = $this->accessos->consultar_accessos_filtre($camp,$valor);		
  	  	if(isset($accessos)) {
			$this->view->assign('accessos',$accessos);
  		}
  		
  		// missatge de consulta filtrada d'usuaris
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Accessos que compleixen la cerca <i>".$camp." = '%".$valor."%'</i>. <br>Per tornar a veure tots els usuaris, fer una <i>cerca</i> buida.";
  		$this->view->assign('missatge',$missatge);
  		
		$this->view->display('accessos_consulta_view');
  	}  
  }

  /*
   * Funció que mostra les dades d'un accés determinat a la vista corresponent
   */
  function consultaAcces() {
  	$this->sessioValida();
  	// es recupera l'agrupació d'usuaris i de servidors que conformen un determinat accés
  	$ugrups = $_POST['ugroup'];
  	$sgrups = $_POST['sgroup'];
	
    $this->load->model('Accessos_Model','accessos',null,'default');
  	$acces = $this->accessos->consultar_acces($ugrups,$sgrups);
  	
  	// missatge inicial consulta accés
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Es pot <i>modificar</i> la data d'expiraci&oacute; de l'acc&eacute;s i desar els canvis amb <img src='/tinymvc/myapp/views/img/accept.png'><br> o b&eacute; <i>eliminar-lo</i> amb <img src='/tinymvc/myapp/views/img/delete.png'>.";
	$this->view->assign('missatge',$missatge);
  	//$grup['name']
    $this->view->assign('acces',$acces);
  	
	$this->view->display('acces_consulta_view');
  }
  
  /*
   * Funció que permet modificar les dades d'un accés determinat
   */
  function modificaAcces() {
  	$this->sessioValida();
  	// es recuperen els paràmetres de l'accés passats per POST
  	$ugroup = $_POST['ugroup'];
  	$sgroup = $_POST['sgroup'];

  	$dataexp = date('d-m-y',strtotime($_POST['dataexp']));
  	$datanull = date('d-m-y',strtotime('01-01-70'));
  	// es controla el valor del camp dataexp, que serà null en cas que no 
  	// se li assigni cap valor
  	if ($dataexp == $datanull) {
  		$dataexp = null;
  	} 		
  	
  	$this->load->model('Accessos_Model','accessos',null,'default');

  	$this->accessos->modificar_acces($ugroup,$sgroup,$dataexp);
  	$this->registrar('Modificada data expiracio acces grup usuari ['.$ugroup.'] a grup servidor ['.$sgroup.']');
  	
  	// es torna a mostrar la vista anterior de consulta de tots els accessos, 
  	// amb el que s'hi assignen les variables necessàries per la generació
  	// de la vista al complet. No s'ha fet una crida directa a la funció
  	// consultaAccessos(), ja que es vol mostrar el missatge corresponent de
  	// modificació ok de l'accés 
  	$this->load->model('Accessos_Model','accessos',null,'default');
  	$accessos = $this->accessos->consultar_accessos();
	$this->view->assign('accessos',$accessos);
	
	// missatge de modificació correcta de l'usuari
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Acc&eacute;s grup usuari ['.$ugroup.'] a grup servidor ['.$sgroup.'] modificat correctament.";
	$this->view->assign('missatge',$missatge);
		
	$this->view->display('accessos_consulta_view');
  }
  
  /*
   * Funció que dóna de baixa un accés determinat
   */
  function baixaAcces() {
  	$this->sessioValida();
  	// es recuperen els paràmetres de l'accés passats per POST
  	$ugroup = $_POST['ugroup'];
  	$sgroup = $_POST['sgroup'];
  	
  	$this->load->model('Accessos_Model','accessos',null,'default');
  	$this->accessos->esborrar_acces($ugroup,$sgroup);
  	$this->registrar('Baixa acces grup usuari  ['.$ugroup.'] a grup servidor ['.$sgroup.'] modificat correctament.');
    
  	// es torna a mostrar la vista anterior de consulta de tots els accessos, 
  	// amb el que s'hi assignen les variables necessàries per la generació
  	// de la vista al complet. No s'ha fet una crida directa a la funció
  	// consultaAccessos(), ja que es vol mostrar el missatge corresponent de
  	// modificació ok de l'accés 
  	$this->load->model('Accessos_Model','accessos',null,'default');
  	$accessos = $this->accessos->consultar_accessos();
	$this->view->assign('accessos',$accessos);
			
	// missatge d'eliminació correcta de l'accés
	$missatge['tipus']="msg-ok";
	$missatge['text']="Baixa acc&eacute;s grup usuari  ['.$ugroup.'] a grup servidor ['.$sgroup.'] modificat correctament.'";
	$this->view->assign('missatge',$missatge);
			
	$this->view->display('accessos_consulta_view');	
  }  
  
}
?>