<?php
class Ram_Controller extends TinyMVC_Controller
{

  /******************************
   * Subsistema d'identificació *
   ******************************/
	
  /**************************
   * Gestió d'identificació *
   **************************/
	
  /*
   * Funció d'inici de l'aplicació: si l'usuari està autenticat, mostra
   * el menú inicial; en cas contrari, mostra la pantalla inicial de login
   */
  function index() {
 	// s'inicia la sessió
  	session_start();
  	
  	if (isset($_SESSION['autenticat']) and $_SESSION['autenticat']==1) {
  		$this->view->display('main_view');
  	} else {
  		$_SESSION['autenticat']=0;
  		
  		// missatge inicial validació usuari
  		$missatge['tipus']="msg-ok";
	  	$missatge['text']="Validaci&oacute; a l'aplicaci&oacute;.";
		$this->view->assign('missatge',$missatge);
		
		$this->view->display('main_login_view');
  	}
  } 
  
  /*
   * Funció de desconnexió de l'aplicació: elimina la sessió existent.
   */
  function disconnect() {
  	
    session_start();
    session_unset();
    session_destroy();
    
	$missatge['tipus']="msg-ok";
  	$missatge['text']="Desconnexi&oacute; realitzada correctament.";
	$this->view->assign('missatge',$missatge);	
	$this->view->display('main_login_view');
  }

  /*
   * Funció que gestiona la validació (login) de l'usuari a l'aplicació
   */
  function loginUsuari() {
  	$username=$_POST['username'];
  	$password=$_POST['password'];
  	
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$admin = $this->usuaris->isAdmin($username);
  	
  	// si és administrador i el perfil està inicialitzat, permetem que faci login
  	if (isset($admin[0]['admin']) and $admin[0]['admin'] == 1) {
  	
	  	$validacio = $this->usuaris->validar_usuari($username,md5($password));
	  	
	  	if ($validacio[0]['validacio']==1) {
	  		session_start();
	  		$_SESSION['autenticat']=1;
	  		$_SESSION['username']=$username;
	  		$_SESSION['admin']=1;
	
			$this->view->display('main_view');
			
	  	} else {
	  		
	  		 // missatge d'error en el procés de validació de l'usuari
	  		$missatge['tipus']="msg-warn";
		  	$missatge['text']="Validaci&oacute; incorrecta: revisa el <i>username</i> i/o la <i>password</i>.";
			$this->view->assign('missatge',$missatge);
			
	  		$this->view->display('main_login_view');
	  	}   
  	} else {
   		 // missatge d'error d'usuari no autoritzat
  		$missatge['tipus']="msg-warn";
	  	$missatge['text']="Usuari no autoritzat per accedir a l'aplicaci&oacute;.";
		$this->view->assign('missatge',$missatge);
		
  		$this->view->display('main_login_view'); 		
  	}
  } 
  
  /*
   * Funció que redirigeix a la vista de canvi de contrasenya d'un usuari
   */
  function canviPasswordVista() {
	$missatge['tipus']="msg-ok";
	$missatge['text']="Canvi de password. Els camps marcats amb <img src='/tinymvc/myapp/views/img/asterisk_orange.png'> s&oacute;n obligatoris";
	$this->view->assign('missatge',$missatge);
  	$this->view->display('main_canvi_password_view');
  }

  /*
   * Funció que permet canviar la contrasenya d'un usuari administrador
   */
  function canviPassword() {
  	
  	$username=$_POST['username'];
  	$password=$_POST['password'];
  	$password_nova=$_POST['password1'];
  	$password_nova_confirmacio=$_POST['password2'];
   	
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$validacio = $this->usuaris->validar_usuari($username,md5($password));
  	
  	if ($validacio[0]['validacio']==1) {
  		if (($password_nova == $password_nova_confirmacio) and $password_nova!="")  {
  			// es canvia la password
  			$this->usuaris->canviar_password($username,md5($password_nova));
  			$missatge['tipus']="msg-ok";
	  		$missatge['text']="Password canviada correctament.";
			$this->view->assign('missatge',$missatge);
			$this->view->display('main_login_view');
		
		// es controla el cas que la password i/o la confirmació no es deixin buides
  		} else if (trim($password_nova) =="" or trim($password_nova_confirmacio="")) {
			$missatge['tipus']="msg-warn";
	  		$missatge['text']="La nova password i/o la confirmaci&oacute; no poden estar en blanc<br>si es vol canviar la <i>password</i>.";
			$this->view->assign('missatge',$missatge);
			$this->view->display('main_canvi_password_view');  	
					
  		} else {
  			$missatge['tipus']="msg-warn";
	  		$missatge['text']="La nova password i la confirmaci&oacute; no coincideixen.";
			$this->view->assign('missatge',$missatge);
			$this->view->display('main_canvi_password_view');
  		}
  	} else {
  		
		$missatge['tipus']="msg-warn";
	  	$missatge['text']="Usuari i/o password incorrectes.";
		$this->view->assign('missatge',$missatge);
		$this->view->display('main_canvi_password_view');
  		
  	}
  }
}
?>