<?php
class Servidor_Controller extends TinyMVC_Controller
{
  
  /***************************
   * Subsistema de servidors *
   ***************************/
	
  /***********************
   * Gestió de servidors *
   ***********************/
	
  /*
   * Funció que indica si l'usuari administrador està autenticat o no. 
   * Aquesta funció s'utilitza internament per validar si es pot executar o no
   * una determinada acció.
   */
  private function sessioValida() {
    session_start();
  	if (!isset($_SESSION['autenticat']) or $_SESSION['autenticat']==0) {
		header('location:../ram/disconnect');
  	}
  }
  
  /*
   * Funció que s'utilitza per actualitzar el log intern de l'aplicació
   */
  private function registrar($descripcio) {
    //session_start();
  	$username=$_SESSION['username'];
  	$this->load->model('Logs_Model','logs',null,'default');
  	$this->logs->registrar($username,$descripcio);
  }

  /*
   * Funció que redirigeix a la vista d'alta de servidor; prèviament
   * s'han recopilat totes les dades necessàries per la vista.
   */
  function altaServidorVista() {
  	$this->sessioValida();
  	// missatge inicial alta servidor
  	$missatge['tipus']="msg-ok";
  	$missatge['text']="Donar d'alta un servidor al sistema. Els camps obligatoris estan marcats amb <img src='/tinymvc/myapp/views/img/asterisk_orange.png'>";
  	$this->view->assign('missatge',$missatge);
  
  	// s'obtenen els grups donats d'alta
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	$grups=$this->servidors->consultar_grups();
  	$this->view->assign('grups',$grups);

    $this->view->display('servidor_alta_view');
  }
  
  /*
   * Funció que dóna d'alta un servidor a l'aplicació
   */
  function altaServidor() {
	$this->sessioValida();
  	// es recuperen les dades del servidor enviades per POST
  	$hostname = $_POST['hostname'];
  	$ip = $_POST['ip'];
  	
    if (isset($_POST['mixedmode'])) {
  		$mixedmode = 1;
  	} else {
  		$mixedmode = 0;
  	}
  	
  	$knownhost = $_POST['knownhost'];
  	$user = $_POST['user'];
  	$path = $_POST['path'];
  	$descripcio = $_POST['descripcio'];
  	if (isset($_POST['grups'])) {
		$grups = $_POST['grups'];
  	}
  	
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	
  	// es controla a nivell d'aplicació que no es deixi buit el camp hostname
  	if (trim($hostname)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="El valor <i>hostname</i> no pot ser buit";
  		
  	} else if (trim($user)=="" or trim($path)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="&Eacute;s necessari indicar l'<i>usuari</i> dest&iacute; i el seu <i>path</i>.";
  		
  	} else {
  		// es dóna d'alta el servidor
  		$this->servidors->insertar_servidor($hostname,$ip,$mixedmode,$knownhost,$user,$path,$descripcio);
  		$this->registrar('Alta servidor ['.$hostname.']');
  		
  		// es dóna d'alta el grup individual corresponent
  		$this->servidors->insertar_grup($hostname,"grup individual",1);
  		$this->registrar('Alta grup servidor individual ['.$hostname.']');
  	
		// es dóna d'alta l'associacio servidor - grup de servidors
  		$this->servidors->insertar_agrupacio($hostname,$hostname);

  		if (isset($_POST['grups'])) {
			for ($i=0; $i<count($grups);$i++) {
				$this->servidors->insertar_agrupacio($hostname,$grups[$i]);
				$this->registrar('Associat servidor ['.$hostname.'] a grup ['.$grups[$i].']');
			}
  		}

  		// missatge alta ok
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Servidor '".$hostname."' donat d'alta correctament.";
  	}
  	
  	// s'actualitza el missatge amb el resultat de l'insert a la vista corresponent
  	$this->view->assign('missatge',$missatge);
  	
  	// es passa a la vista informació relativa als grups disponibles
  	$grups=$this->servidors->consultar_grups();
  	$this->view->assign('grups',$grups);
  	
    $this->view->display('servidor_alta_view');
  }
  
  /*
   * Funció que consulta tots els servidors donats d'alta a l'aplicació
   */
  function consultaServidors() {
  	$this->sessioValida();
  	// cas 1: estem davant d'una cerca genèrica, que és la vista inicial
  	// que es presenta o bé el servidor ha forçat aquesta vista.
  	if (!isset($_POST['cerca']) or $_POST['cerca']=="") {
 
		$this->load->model('Servidors_Model','servidors',null,'default');
  		$servidors = $this->servidors->consultar_servidors();
		$this->view->assign('servidors',$servidors);
		
		// missatge inicial consulta servidors
  		$missatge['tipus']="msg-ok";
	  	$missatge['text']="Consulta de servidors. Es pot filtrar la sortida amb l'opci&oacute; <i>cercar</i>.";
		$this->view->assign('missatge',$missatge);
		
		$this->view->display('servidors_consulta_view');

	// cas 2: el servidor ha escollit fer una cerca, de forma que s'ha de filtrar
	// la sortida que es mostrarà a la vista corresponent
  	} else {
  		
  		// es recuperen els paràmetres de la cerca passats per POST
  		$valor=$_POST['cerca'];
  		$camp=$_POST['opcio'];
  		
  		$this->load->model('Servidors_Model','servidors',null,'default');

  		// en cas que no s'hagi trobat cap servidor que compleixi la cerca, 
  		// no es retorna cap valor (=null) a la variable servidors de la vista
  		$servidors = $this->servidors->consultar_servidors_filtre($camp,$valor);		
  	  	if(isset($servidors)) {
			$this->view->assign('servidors',$servidors);
  		}
  		
  		// missatge de consulta filtrada de servidors
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Servidors que compleixen la cerca <i>".$camp." = '%".$valor."%'</i>. <br>Per tornar a veure tots els servidors, fer una <i>cerca</i> buida.";
  		$this->view->assign('missatge',$missatge);
  		
		$this->view->display('servidors_consulta_view');
  	}  
  }
  
  /*
   * Funció que mostra a la vista corresponent els atributs d'un determinat servidor
   */
  function consultaServidor() {
  	$this->sessioValida();
  	// es recupera el hostname passat per POST
  	$hostname = $_POST['hostname'];
	
    $this->load->model('Servidors_Model','servidors',null,'default');
  	$servidor = $this->servidors->consultar_servidor($hostname);
  	
  	// missatge inicial consulta servidors
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Es pot <i>modificar</i> el servidor i desar els canvis amb <img src='/tinymvc/myapp/views/img/accept.png'> (els camps <img src='/tinymvc/myapp/views/img/asterisk_orange.png'> s&oacute;n obligatoris)<br> o b&eacute; <i>eliminar-lo</i> amb <img src='/tinymvc/myapp/views/img/delete.png'>.";
	$this->view->assign('missatge',$missatge);
  	//$grup['name']
    $this->view->assign('servidor',$servidor);
    
    // es passa a la vista informació relativa als grups disponibles
  	$grups=$this->servidors->consultar_grups();
  	$this->view->assign('grups',$grups);
  
  	$checkeds="";
 	for ($i=0; $i<count($grups);$i++) {
	  	$checkeds[$grups[$i]['name']]=$this->isChecked($hostname, $grups[$i]['name']);
	}
	
  	$this->view->assign('checkeds',$checkeds);
  	
	$this->view->display('servidor_consulta_view');
  }
  
  /*
   * Funció interna que indica si existeix una agrupació entre el servidor
   * $hostname i el grup de servidors $groupname 
   */
  private function isChecked($hostname,$groupname){
	$this->load->model('Servidors_Model','servidors',null,'default');
	$isChecked=$this->servidors->isChecked_grup($hostname,$groupname);
	if ($isChecked[0]['numChecked']==1) {
		return 'yes';
	} else {
		return 'no';
	}
  }
  
  /*
   * Funció que gestiona la modificació d'un servidor determinat
   */
  function modificaServidor() {
  	$this->sessioValida();
  	// es recuperen els paràmetres del servidor passats per POST
  	$hostname = $_POST['hostname'];
  	$ip = $_POST['ip'];
  	$descripcio = $_POST['descripcio'];

    if (isset($_POST['mixedmode'])) {
  		$mixedmode = 1;
  	} else {
  		$mixedmode = 0;
  	}
  	
  	$knownhost = $_POST['knownhost'];
  	$user = $_POST['user'];
  	$path = $_POST['path'];
  	
  	if (isset ($_POST['grups'])){
  		$grups=$_POST['grups'];
  	}
  	
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	
  	// es controla que el camp obligatori no estigui buit
  	if (trim($hostname)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="El camp <i>clau p&uacute;blica</i> no pot ser buit";
  		$this->view->assign('missatge',$missatge);
  		$servidor[0]['hostname']=$hostname;
  		$servidor[0]['ip']=$ip;
  		$servidor[0]['descripcio']=$descripcio;
  		$servidor[0]['mixedmode']=$mixedmode;
  		$servidor[0]['knownhost']=$knownhost;
  		$servidor[0]['user']=$user;
  		$servidor[0]['path']=$path;  		
  		
  		$this->view->assign('servidor',$servidor);
		$this->view->display('servidor_consulta_view');
	
  	} else if (trim($user)=="" or trim($path)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="&Eacute;s necessari indicar l'<i>usuari</i> dest&iacute; i el seu <i>path</i>.";
  		
  		$servidor[0]['hostname']=$hostname;
  		$servidor[0]['ip']=$ip;
  		$servidor[0]['descripcio']=$descripcio;
  		$servidor[0]['mixedmode']=$mixedmode;
  		$servidor[0]['knownhost']=$knownhost;
  		$servidor[0]['user']=$user;
  		$servidor[0]['path']=$path;  
  		
  		$this->view->assign('servidor',$servidor);
		$this->view->display('servidor_consulta_view');
  		
  	} else {	
    	
  		$servidor = $this->servidors->modificar_servidor($hostname,$ip,$mixedmode,$knownhost,$user,$path,$descripcio);
  		$this->registrar('Modificat servidor ['.$hostname.']');
  		
  		// s'elimina el totes les assignacions prèvies fetes del servidor 
  		// a qualsevol grup no individual
		$this->servidors->esborrar_agrupacions($hostname);
		$this->registrar('Desassociat hostname ['.$hostname.'] de tots els grups');
		
		// i a continuació es donen d'alta les noves agrupacions obtingudes
		// des del formulari
		if (isset ($_POST['grups'])) {
  	 		for ($i=0; $i<count($grups);$i++) {
				$this->servidors->insertar_agrupacio($hostname,$grups[$i]);
				$this->registrar('Associat hostname ['.$hostname.'] a grup ['.$grups[$i].']');
			}
		}
  	
  		// es torna a mostrar la vista anterior de consulta de tots els servidors, 
	  	// amb el que s'hi assignen les variables necessàries per la generació
	  	// de la vista al complet. No s'ha fet una crida directa a la funció
	  	// consultaServidors(), ja que es vol mostrar el missatge corresponent de
	  	// modificació ok de servidor 
	  	$this->load->model('Servidors_Model','servidors',null,'default');
	  	$servidors = $this->servidors->consultar_servidors();
		$this->view->assign('servidors',$servidors);
		
		// missatge de modificació correcta del servidor
	  	$missatge['tipus']="msg-ok";
		$missatge['text']="Servidor '".$hostname."' modificat correctament.";
		$this->view->assign('missatge',$missatge);
			
		$this->view->display('servidors_consulta_view');
  	}
  }
  
  /* 
   * Funció que gestiona la baixa d'un determinat servidor
   */
  function baixaServidor() {
  	$this->sessioValida();
  	$hostname = $_POST['hostname'];
  	
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	$this->servidors->esborrar_servidor($hostname);
  	$this->registrar('Baixa servidor ['.$hostname.']');
    
    // es torna a mostrar la vista anterior de consulta de tots els servidors, 
	// amb el que s'hi assignen les variables necessàries per la generació
	// de la vista al complet. No s'ha fet una crida directa a la funció
	// consultaServidors(), ja que es vol mostrar el missatge corresponent de
	// modificació ok de servidor 
	$this->load->model('Servidors_Model','servidors',null,'default');
	$servidors = $this->servidors->consultar_servidors();
	$this->view->assign('servidors',$servidors);
			
	// missatge de modificació correcta del servidor
	$missatge['tipus']="msg-ok";
	$missatge['text']="Servidor '".$hostname."' eliminat correctament.";
	$this->view->assign('missatge',$missatge);
	
	// s'elimina el totes les assignacions prèvies fetes del servidor a qualsevol
	// grup, i a més també la corresponent al grup individual
	$this->servidors->esborrar_agrupacions($hostname);
	$this->registrar('Baixa associacions ['.$hostname.']');
	$this->servidors->esborrar_agrupacio($hostname,$hostname);
	$this->registrar('Baixa grup individual ['.$hostname.']');
			
	$this->view->display('servidors_consulta_view');	
  }
  
  /********************************
   * Gestió de grups de servidors *
   ********************************/
  
  /*
   * Funció que redirecciona a la vista d'alta de grup de servidors
   */
  function altaGrupVista() {
  	$this->sessioValida();
  	
  	// missatge inicial alta grup de servidors
  	$missatge['tipus']="msg-ok";
  	$missatge['text']="Donar d'alta un grup de servidors al sistema. Els camp obligatori est&agrave; <br>marcat amb <img src='/tinymvc/myapp/views/img/asterisk_orange.png'>.";
  	
  	$this->view->assign('missatge',$missatge);
    $this->view->display('sgrup_alta_view');

  }  

  /*
   * Funció que gestiona l'alta d'un grup de servidors concret
   */
  function altaGrup() {
  	$this->sessioValida();
  	// es recuperen les dades del servidor enviades per POST
	$nom = $_POST['nom'];
  	$descripcio = $_POST['descripcio'];
  	
  	// es controla a nivell d'aplicació que no deixi buit el camp nom
  	if (trim($nom)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="El camp <i>nom</i> no pot ser buit";
  	} else {
  		// es dóna d'alta el grup de servidors
  		$this->load->model('Servidors_Model','servidors',null,'default');
  		$this->servidors->insertar_grup($nom,$descripcio,0);
  		$this->registrar('Alta grup servidor ['.$nom.']');
  		// missatge alta ok
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Grup de servidors '".$nom."' donat d'alta correctament.";
  	}
  	
  	// s'actualitza el missatge amb el resultat de l'insert a la vista corresponent
  	$this->view->assign('missatge',$missatge);
    $this->view->display('sgrup_alta_view');
    
  }
  
  /*
   * Funció que mostra tots els grups de servidors de l'aplicació
   */
  function consultaGrups() {
  	$this->sessioValida();
  	// cas 1: estem davant d'una cerca genèrica, que és la vista inicial
  	// que es presenta o bé el servidor ha forçat aquesta vista.
  	if (!isset($_POST['cerca']) or $_POST['cerca']=="") {
 
		$this->load->model('Servidors_Model','servidors',null,'default');
  		$grups = $this->servidors->consultar_grups();
		$this->view->assign('grups',$grups);
		
		// missatge inicial consulta grups
  		$missatge['tipus']="msg-ok";
	  	$missatge['text']="Consulta de grups. Es pot filtrar la sortida amb l'opci&oacute; <i>cercar</i>.";
		$this->view->assign('missatge',$missatge);
		
		$this->view->display('sgrups_consulta_view');

	// cas 2: el servidor ha escollit fer una cerca, de forma que s'ha de filtrar
	// la sortida que es mostrarà a la vista corresponent
  	} else {
  		
  		// es recuperen els paràmetres de la cerca passats per POST
  		$valor=$_POST['cerca'];
  		$camp=$_POST['opcio'];
  		
  		$this->load->model('Servidors_Model','servidors',null,'default');

  		// en cas que no s'hagi trobat cap grup que compleixi la cerca, 
  		// no es retorna cap valor (=null) a la variable servidors de la vista
  		$grups = $this->servidors->consultar_grups_filtre($camp,$valor);	
  		
  	  	if(isset($grups)) {
			$this->view->assign('grups',$grups);
  		}
  		
  		// missatge de consulta filtrada de grups
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Grups que compleixen la cerca <i>".$camp." = '%".$valor."%'</i>. <br>Per tornar a veure tots els servidors, fer una <i>cerca</i> buida.";
  		$this->view->assign('missatge',$missatge);
  		
		$this->view->display('sgrups_consulta_view');
  	}  
  }
  
  /*
   * Funció que mostra totes les dades d'un determinat grup de servidors
   */
  function consultaGrup() {
  	$this->sessioValida();
  	// es recupera el paràmetre de la cerca passat per POST
  	$name = $_POST['name'];
	
    $this->load->model('Servidors_Model','servidors',null,'default');
  	$grup = $this->servidors->consultar_grup($name);
  	
  	// missatge inicial consulta grups
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Es pot <i>modificar</i> el grup i desar els canvis amb <img src='/tinymvc/myapp/views/img/accept.png'><br> o b&eacute; <i>eliminar-lo</i> amb <img src='/tinymvc/myapp/views/img/delete.png'>.";
	$this->view->assign('missatge',$missatge);
  	
    $this->view->assign('grup',$grup);
	$this->view->display('sgrup_consulta_view');
  }
  
  /*
   * Funció que gestiona la modificació de dades d'un grup de servidors determinat
   */
  function modificaGrup() {
  	$this->sessioValida();
  	// es recuperen els paràmetres del servidor passats per POST
  	$nom = $_POST['nom'];
  	$descripcio = $_POST['descripcio'];
  	  	
   	$this->load->model('Servidors_Model','servidors',null,'default');
	$grup = $this->servidors->modificar_grup($nom,$descripcio);
	$this->registrar('Modificat grup servidor ['.$nom.']');
  	
	// es torna a mostrar la vista anterior de consulta de tots els grups, 
 	// amb el que s'hi assignen les variables necessàries per la generació
  	// de la vista al complet. No s'ha fet una crida directa a la funció
  	// consultaGrups(), ja que es vol mostrar el missatge corresponent de
  	// modificació ok de grup. 
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	$grups = $this->servidors->consultar_grups();
	$this->view->assign('grups',$grups);
			
	// missatge de modificació correcta del grup
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Grup '".$nom."' modificat correctament.";
	$this->view->assign('missatge',$missatge);
		
	$this->view->display('sgrups_consulta_view');
  }  
  
  /*
   * Funció que gestiona la baixa d'un determinat grup de servidors
   */
  function baixaGrup() {
  	$this->sessioValida();
	$nom = $_POST['nom'];
  	
  	$this->load->model('Servidors_Model','servidors',null,'default');
  	$this->servidors->esborrar_grup($nom);
  	$this->registrar('Baixa grup servidor ['.$nom.']');
  	
  	$this->servidors->esborrar_agrupacions_grup($nom);
  	$this->registrar('Baixa associacions grup servidor ['.$nom.']');
    
    // es torna a mostrar la vista anterior de consulta de tots els grups, 
	// amb el que s'hi assignen les variables necessàries per la generació
	// de la vista al complet. No s'ha fet una crida directa a la funció
	// consultaGrups(), ja que es vol mostrar el missatge corresponent de
	// modificació ok de grups 
	$this->load->model('Servidors_Model','servidors',null,'default');
	$grups = $this->servidors->consultar_grups();
	$this->view->assign('grups',$grups);
			
	// missatge de modificació correcta del servidor
	$missatge['tipus']="msg-ok";
	$missatge['text']="Grup '".$nom."' eliminat correctament.";
	$this->view->assign('missatge',$missatge);

	$this->view->display('sgrups_consulta_view');
  }
}
?>