<?php
class Usuari_Controller extends TinyMVC_Controller
{
  
  /************************
   * Subsistema d'usuaris *
   ************************/
	
  /********************
   * Gestió d'usuaris *
   ********************/
	
  /*
   * Funció que indica si l'usuari administrador està autenticat o no. 
   * Aquesta funció s'utilitza internament per validar si es pot executar o no
   * una determinada acció.
   */
  private function sessioValida() {
    session_start();
  	if (!isset($_SESSION['autenticat']) or $_SESSION['autenticat']==0) {
		header('location:../ram/disconnect');
  	}
  }
  
  /*
   * Funció que s'utilitza per actualitzar el log intern de l'aplicació
   */
  private function registrar($descripcio) {
  	$username=$_SESSION['username'];
  	$this->load->model('Logs_Model','logs',null,'default');
  	$this->logs->registrar($username,$descripcio);
  }
	
  /*
   * Funció que redirecciona a la vista d'alta d'usuari; prèviament s'ocupa
   * de recollir totes les dades requerides per la vista
   */
  function altaUsuariVista() {
  	$this->sessioValida();
  	// missatge inicial alta usuari
  	$missatge['tipus']="msg-ok";
  	$missatge['text']="Donar d'alta un usuari al sistema. Els camps obligatoris estan marcats amb <img src='/tinymvc/myapp/views/img/asterisk_orange.png'>";
  	$this->view->assign('missatge',$missatge);
  
  	// s'obtenen els grups donats d'alta
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$grups=$this->usuaris->consultar_grups();
  	$this->view->assign('grups',$grups);

    $this->view->display('usuari_alta_view');
  }
  
  /*
   * Funció que gestiona l'alta d'un usuari a l'aplicació
   */
  function altaUsuari() {
	$this->sessioValida();
  	// es recuperen les dades de l'usuari enviades per POST
  	$username = $_POST['username'];
  	$nom = $_POST['nom'];
  	$cognoms = $_POST['cognoms'];
    if (isset($_POST['admin'])) {
  		$admin = 1;
  	} else {
  		$admin = 0;
  	}
  	$password = $_POST['password1'];
  	$password_confirmacio = $_POST['password2'];
  	$descripcio = $_POST['descripcio'];
  	$clauPublica = $_POST['clauPublica'];
  	if (isset($_POST['grups'])) {
		$grups = $_POST['grups'];
  	}
  	
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	
  	// es controla a nivell d'aplicació que no deixin buits els
  	// camps username i/o clauPublica
  	if (trim($username)=="" or trim($clauPublica)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="Els valors <i>username</i> i/o <i>clau p&uacute;blica</i> no poden ser buits";
  		
  	} else if (trim($password)=="" or trim($password_confirmacio)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="Tant la password com la confirmaci&oacute; no poden ser buides.";
  		
  	} else if ($password != $password_confirmacio) {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="La password i la confirmaci&oacute; no coincideixen.";
  		  		
  	} else {
  		// es dóna d'alta l'usuari
  		$this->usuaris->insertar_usuari($username,$nom,$cognoms,$admin,md5($password),$descripcio,$clauPublica);
  		$this->registrar('Alta usuari ['.$username.']');
  		
  		// es dóna d'alta el grup individual corresponent
  		$this->usuaris->insertar_grup($username,"grup individual",0,1);
  		$this->registrar('Alta grup usuari individual ['.$username.']');
  	
		// es dóna d'alta l'associacio usuari - grup d'usuaris
  		$this->usuaris->insertar_agrupacio($username,$username);

  		if (isset($_POST['grups'])) {
			for ($i=0; $i<count($grups);$i++) {
				$this->usuaris->insertar_agrupacio($username,$grups[$i]);
				$this->registrar('Associat usuari ['.$username.'] a grup ['.$grups[$i].']');
			}
  		}

  		// missatge alta ok
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Usuari '".$username."' donat d'alta correctament.";
  	}
  	
  	// s'actualitza el missatge amb el resultat de l'insert a la vista corresponent
  	$this->view->assign('missatge',$missatge);
  	
  	// es passa a la vista informació relativa als grups disponibles
  	$grups=$this->usuaris->consultar_grups();
  	$this->view->assign('grups',$grups);
  	
    $this->view->display('usuari_alta_view');
  }
  
  /* 
   * Funció que retorna tots els usuaris donats d'alta
   */
  function consultaUsuaris() {
  	$this->sessioValida();
  	// cas 1: estem davant d'una cerca genèrica, que és la vista inicial
  	// que es presenta o bé l'usuari ha forçat aquesta vista.
  	if (!isset($_POST['cerca']) or $_POST['cerca']=="") {
 
		$this->load->model('Usuaris_Model','usuaris',null,'default');
  		$usuaris = $this->usuaris->consultar_usuaris();
		$this->view->assign('usuaris',$usuaris);
		
		// missatge inicial consulta usuaris
  		$missatge['tipus']="msg-ok";
	  	$missatge['text']="Consulta d'usuaris. Es pot filtrar la sortida amb l'opci&oacute; <i>cercar</i>.";
		$this->view->assign('missatge',$missatge);
		
		$this->view->display('usuaris_consulta_view');

	// cas 2: l'usuari ha escollit fer una cerca, de forma que s'ha de filtrar
	// la sortida que es mostrarà a la vista corresponent
  	} else {
  		
  		// es recuperen els paràmetres de la cerca passats per POST
  		$valor=$_POST['cerca'];
  		$camp=$_POST['opcio'];
  		
  		$this->load->model('Usuaris_Model','usuaris',null,'default');

  		// en cas que no s'hagi trobat cap usuari que compleixi la cerca, 
  		// no es retorna cap valor (=null) a la variable usuaris de la vista
  		$usuaris = $this->usuaris->consultar_usuaris_filtre($camp,$valor);		
  	  	if(isset($usuaris)) {
			$this->view->assign('usuaris',$usuaris);
  		}
  		
  		// missatge de consulta filtrada d'usuaris
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Usuaris que compleixen la cerca <i>".$camp." = '%".$valor."%'</i>. <br>Per tornar a veure tots els usuaris, fer una <i>cerca</i> buida.";
  		$this->view->assign('missatge',$missatge);
  		
		$this->view->display('usuaris_consulta_view');
  	}  
  }
  
  /*
   * Funció que retorna les dades d'un usuari concret
   */
  function consultaUsuari() {
  	$this->sessioValida();
  	// es recupera el username passat per POST
  	$username = $_POST['username'];
	
    $this->load->model('Usuaris_Model','usuaris',null,'default');
  	$usuari = $this->usuaris->consultar_usuari($username);
  	
  	// missatge inicial consulta usuaris
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Es pot <i>modificar</i> l'usuari i desar els canvis amb <img src='/tinymvc/myapp/views/img/accept.png'> (els camps <img src='/tinymvc/myapp/views/img/asterisk_orange.png'> s&oacute;n obligatoris)<br> o b&eacute; <i>eliminar-lo</i> amb <img src='/tinymvc/myapp/views/img/delete.png'>.";
	$this->view->assign('missatge',$missatge);
  	//$grup['name']
    $this->view->assign('usuari',$usuari);
    
    // es passa a la vista informació relativa als grups disponibles
  	$grups=$this->usuaris->consultar_grups();
  	$this->view->assign('grups',$grups);
  
  	$checkeds="";
 	for ($i=0; $i<count($grups);$i++) {
	  	$checkeds[$grups[$i]['name']]=$this->isChecked($username, $grups[$i]['name']);
	}
	
  	$this->view->assign('checkeds',$checkeds);
  	
	$this->view->display('usuari_consulta_view');
  }
  
  /*
   * Funció que indica si existeix o no una agrupació
   * entre l'usuari $username i el grup d'usuaris $groupname
   */
  private function isChecked($username,$groupname){
	$this->load->model('Usuaris_Model','usuaris',null,'default');
	$isChecked=$this->usuaris->isChecked_grup($username,$groupname);
	if ($isChecked[0]['numChecked']==1) {
		return 'yes';
	} else {
		return 'no';
	}
  }
  
  /*
   * Funció que gesiona la modificació d'un usuari determinat
   */
  function modificaUsuari() {
  	$this->sessioValida();
  	// es recuperen els paràmetres de l'usuari passats per POST
  	$username = $_POST['username'];
  	$nom = $_POST['nom'];
  	$cognoms = $_POST['cognoms'];
    if (isset($_POST['admin'])) {
  		$admin = 1;
  	} else {
  		$admin = 0;
  	}
  	$descripcio = $_POST['descripcio'];
  	$clauPublica = $_POST['clauPublica'];
  	if (isset ($_POST['grups'])){
  		$grups=$_POST['grups'];
  	}
  	
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	
  	// es controla que els dos camps obligatoris no estiguin buits
  	if (trim($username)=="" or trim($clauPublica)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="El camp <i>clau p&uacute;blica</i> no pot ser buit";
  		$this->view->assign('missatge',$missatge);
  		$usuari[0]['username']=$username;
  		$usuari[0]['name']=$nom;
  		$usuari[0]['surnames']=$cognoms;
  		$usuari[0]['info']=$descripcio;
  		$usuari[0]['public_key']=$clauPublica;
  		
  		$this->view->assign('usuari',$usuari);
		$this->view->display('usuari_consulta_view');
  		
  	} else {	
    	
  		$usuari = $this->usuaris->modificar_usuari($username,$nom,$cognoms,$admin,$descripcio,$clauPublica);
  		$this->registrar('Modificat usuari ['.$username.']');
  		
  		// s'elimina el totes les assignacions prèvies fetes de l'usuari 
  		// a qualsevol grup no individual
		$this->usuaris->esborrar_agrupacions($username);
		$this->registrar('Desassociat usuari ['.$username.'] de tots els grups');
		
		// i a continuació es donen d'alta les noves agrupacions obtingudes
		// des del formulari
		if (isset ($_POST['grups'])) {
  	 		for ($i=0; $i<count($grups);$i++) {
				$this->usuaris->insertar_agrupacio($username,$grups[$i]);
				$this->registrar('Associat usuari ['.$username.'] a grup ['.$grups[$i].']');
			}
		}
  	
  		// es torna a mostrar la vista anterior de consulta de tots els usuaris, 
	  	// amb el que s'hi assignen les variables necessàries per la generació
	  	// de la vista al complet. No s'ha fet una crida directa a la funció
	  	// consultaUsuaris(), ja que es vol mostrar el missatge corresponent de
	  	// modificació ok d'usuari 
	  	$this->load->model('Usuaris_Model','usuaris',null,'default');
	  	$usuaris = $this->usuaris->consultar_usuaris();
		$this->view->assign('usuaris',$usuaris);
		
		// missatge de modificació correcta de l'usuari
	  	$missatge['tipus']="msg-ok";
		$missatge['text']="Usuari '".$username."' modificat correctament.";
		$this->view->assign('missatge',$missatge);
			
		$this->view->display('usuaris_consulta_view');
  	}
  }
  
  /*
   * Funció que gestiona la baixa d'un determinat usuari
   */
  function baixaUsuari() {
  	$this->sessioValida();
  	$username = $_POST['username'];
  	
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$this->usuaris->esborrar_usuari($username);
  	$this->registrar('Baixa usuari ['.$username.']');
    
    // es torna a mostrar la vista anterior de consulta de tots els usuaris, 
	// amb el que s'hi assignen les variables necessàries per la generació
	// de la vista al complet. No s'ha fet una crida directa a la funció
	// consultaUsuaris(), ja que es vol mostrar el missatge corresponent de
	// modificació ok d'usuari 
	$this->load->model('Usuaris_Model','usuaris',null,'default');
	$usuaris = $this->usuaris->consultar_usuaris();
	$this->view->assign('usuaris',$usuaris);
			
	// missatge de modificació correcta de l'usuari
	$missatge['tipus']="msg-ok";
	$missatge['text']="Usuari '".$username."' eliminat correctament.";
	$this->view->assign('missatge',$missatge);
	
	// s'elimina el totes les assignacions prèvies fetes de l'usuari a qualsevol
	// grup, i a més també la corresponent al grup individual
	$this->usuaris->esborrar_agrupacions($username);
	$this->registrar('Baixa associacions ['.$username.']');
	$this->usuaris->esborrar_agrupacio($username,$username);
	$this->registrar('Baixa grup individual ['.$username.']');
			
	$this->view->display('usuaris_consulta_view');	
  }
  
  /*****************************
   * Gestió de grups d'usuaris *
   *****************************/
  
  /*
   * Funció que redirecciona a la vista d'alta de grups d'usuaris
   */
  function altaGrupVista() {
  	$this->sessioValida();
  	
  	// missatge inicial alta grup d'usuaris
  	$missatge['tipus']="msg-ok";
  	$missatge['text']="Donar d'alta un grup d'usuaris al sistema. Els camp obligatori est&agrave; marcat amb <img src='/tinymvc/myapp/views/img/asterisk_orange.png'><br>Un grup administrador permet administrar els servidors que gestiona.";
  	
  	$this->view->assign('missatge',$missatge);
    $this->view->display('ugrup_alta_view');

  }  

  /*
   * Funció que gestiona l'alta d'un grup d'usuaris a l'aplicació
   */
  function altaGrup() {
  	$this->sessioValida();
  	// es recuperen les dades de l'usuari enviades per POST
	$nom = $_POST['nom'];
  	$descripcio = $_POST['descripcio'];
  	
  	if (isset($_POST['admin'])) {
  		$admin = 1;
  	} else {
  		$admin = 0;
  	}
  	
  	// es controla a nivell d'aplicació que no deixi buit el camp nom
  	if (trim($nom)=="") {
  		// missatge warning
  		$missatge['tipus']="msg-warn";
  		$missatge['text']="El camp <i>nom</i> no pot ser buit";
  	} else {
  		// es dóna d'alta el grup d'usuaris
  		$this->load->model('Usuaris_Model','usuaris',null,'default');
  		$this->usuaris->insertar_grup($nom,$descripcio,$admin,0);
  		$this->registrar('Alta grup usuari ['.$nom.']');
  		// missatge alta ok
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Grup d'usuaris '".$nom."' donat d'alta correctament.";
  	}
  	
  	// s'actualitza el missatge amb el resultat de l'insert a la vista corresponent
  	$this->view->assign('missatge',$missatge);
    $this->view->display('ugrup_alta_view');
    
  }
  
  /*
   * Funció que retorna tots els grups d'usuaris donats d'alta a l'aplicació
   */
  function consultaGrups() {
  	$this->sessioValida();
  	// cas 1: estem davant d'una cerca genèrica, que és la vista inicial
  	// que es presenta o bé l'usuari ha forçat aquesta vista.
  	if (!isset($_POST['cerca']) or $_POST['cerca']=="") {
 
		$this->load->model('Usuaris_Model','usuaris',null,'default');
  		$grups = $this->usuaris->consultar_grups();
		$this->view->assign('grups',$grups);
		
		// missatge inicial consulta grups
  		$missatge['tipus']="msg-ok";
	  	$missatge['text']="Consulta de grups. Es pot filtrar la sortida amb l'opci&oacute; <i>cercar</i>.";
		$this->view->assign('missatge',$missatge);
		
		$this->view->display('ugrups_consulta_view');

	// cas 2: l'usuari ha escollit fer una cerca, de forma que s'ha de filtrar
	// la sortida que es mostrarà a la vista corresponent
  	} else {
  		
  		// es recuperen els paràmetres de la cerca passats per POST
  		$valor=$_POST['cerca'];
  		$camp=$_POST['opcio'];
  		
  		$this->load->model('Usuaris_Model','usuaris',null,'default');

  		// en cas que no s'hagi trobat cap grup que compleixi la cerca, 
  		// no es retorna cap valor (=null) a la variable usuaris de la vista
  		$grups = $this->usuaris->consultar_grups_filtre($camp,$valor);	
  		
  	  	if(isset($grups)) {
			$this->view->assign('grups',$grups);
  		}
  		
  		// missatge de consulta filtrada de grups
  		$missatge['tipus']="msg-ok";
  		$missatge['text']="Grups que compleixen la cerca <i>".$camp." = '%".$valor."%'</i>. <br>Per tornar a veure tots els usuaris, fer una <i>cerca</i> buida.";
  		$this->view->assign('missatge',$missatge);
  		
		$this->view->display('ugrups_consulta_view');
  	}  
  }
  
  /*
   * Funció que retorna el detall d'un grup d'usuaris determinat
   */
  function consultaGrup() {
  	$this->sessioValida();
  	// es recupera el paràmetre de la cerca passat per POST
  	$name = $_POST['name'];
	
    $this->load->model('Usuaris_Model','usuaris',null,'default');
  	$grup = $this->usuaris->consultar_grup($name);
  	
  	// missatge inicial consulta grups
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Es pot <i>modificar</i> el grup i desar els canvis amb <img src='/tinymvc/myapp/views/img/accept.png'><br> o b&eacute; <i>eliminar-lo</i> amb <img src='/tinymvc/myapp/views/img/delete.png'>.";
	$this->view->assign('missatge',$missatge);
  	
    $this->view->assign('grup',$grup);
	$this->view->display('ugrup_consulta_view');
  }
  
  /*
   * Funció que gestiona la modificació d'un grup d'usuaris
   */
  function modificaGrup() {
  	$this->sessioValida();
  	// es recuperen els paràmetres de l'usuari passats per POST
  	$nom = $_POST['nom'];
  	$descripcio = $_POST['descripcio'];
  	
    if (isset($_POST['admin'])) {
  		$admin = 1;
  	} else {
  		$admin = 0;
  	}
  	
   	$this->load->model('Usuaris_Model','usuaris',null,'default');
	$grup = $this->usuaris->modificar_grup($nom,$descripcio,$admin);
	$this->registrar('Modificat grup usuari ['.$nom.']');
  	
	// es torna a mostrar la vista anterior de consulta de tots els grups, 
 	// amb el que s'hi assignen les variables necessàries per la generació
  	// de la vista al complet. No s'ha fet una crida directa a la funció
  	// consultaGrups(), ja que es vol mostrar el missatge corresponent de
  	// modificació ok de grup. 
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$grups = $this->usuaris->consultar_grups();
	$this->view->assign('grups',$grups);
			
	// missatge de modificació correcta del grup
  	$missatge['tipus']="msg-ok";
	$missatge['text']="Grup '".$nom."' modificat correctament.";
	$this->view->assign('missatge',$missatge);
		
	$this->view->display('ugrups_consulta_view');
  }  
  
  /*
   * Funció que gestiona la baixa d'un grup d'usuaris
   */
  function baixaGrup() {
  	$this->sessioValida();
	$nom = $_POST['nom'];
  	
  	$this->load->model('Usuaris_Model','usuaris',null,'default');
  	$this->usuaris->esborrar_grup($nom);
  	$this->registrar('Baixa grup usuari ['.$nom.']');
  	
  	$this->usuaris->esborrar_agrupacions_grup($nom);
  	$this->registrar('Baixa associacions grup usuari ['.$nom.']');
    
    // es torna a mostrar la vista anterior de consulta de tots els grups, 
	// amb el que s'hi assignen les variables necessàries per la generació
	// de la vista al complet. No s'ha fet una crida directa a la funció
	// consultaGrups(), ja que es vol mostrar el missatge corresponent de
	// modificació ok de grups 
	$this->load->model('Usuaris_Model','usuaris',null,'default');
	$grups = $this->usuaris->consultar_grups();
	$this->view->assign('grups',$grups);
			
	// missatge de modificació correcta de l'usuari
	$missatge['tipus']="msg-ok";
	$missatge['text']="Grup '".$nom."' eliminat correctament.";
	$this->view->assign('missatge',$missatge);
			
	$this->view->display('ugrups_consulta_view');
  }
}
?>