<?php
class Servidors_Model extends TinyMVC_Model
{
  
  /**************************
   * Persistència servidors *
   **************************/
	
  /*
   * Funció que retorna un servidor en funció del seu $hostname
   */
  function consultar_servidor($hostname) {
    return $this->db->query_all('select * from Servidors where hostname=?',array($hostname));
  } 
   
  /*
   * Funció que retorna tots els servidors donats d'alta a l'aplicació,
   * ordenats per $hostname.
   */
  function consultar_servidors() {
    return $this->db->query_all('select * from Servidors order by hostname');
  }  
  
  /*
   * Funció que s'utilitza per obtenir el conjunt de servidors que compleixen
   * la condició que el camp $camp contingui el valor $valor.
   */
  function consultar_servidors_filtre($camp,$valor) {
  	
  	switch ($camp) {
  		case "hostname":
  			return $this->db->query_all('select * from Servidors where hostname like ? order by hostname',array('%'.$valor."%"));
  			break;
  		case "ip":
  			return $this->db->query_all('select * from Servidors where ip like ? order by hostname',array('%'.$valor."%"));
  			break;
  		case "info":
  			return $this->db->query_all('select * from Servidors where info like ? order by hostname',array('%'.$valor."%"));
  			break;  			
  	}   
  }
  
  /*
   * Funció que dóna d'alta un servidor a la taula Servidors de l'aplicació
   */
  function insertar_servidor($hostname, $ip, $mixedmode, $knownhost, $user, $path, $info) {
    return $this->db->insert('Servidors',array('hostname'=>$hostname,'ip'=>$ip,'mixed_mode'=>$mixedmode,'hash_known_host'=>$knownhost,'user'=>$user,'path'=>$path,'info'=>$info));
  } 
  
  /*
   * Funció que permet modificar els atributs d'un servidor, en funció
   * del seu $hostname.
   */
  function modificar_servidor($hostname, $ip, $mixedmode, $knownhost, $user, $path, $info) {
  	$this->db->where('hostname',$hostname);
  	return $this->db->update('Servidors',array('ip'=>$ip,'mixed_mode'=>$mixedmode,'hash_known_host'=>$knownhost,'user'=>$user,'path'=>$path,'info'=>$info));
  } 
  
  /*
   * Funció que dóna de baixa un servidor de la taula corresponent 
   * en funció del $hostname facilitat per paràmetre
   */
  function esborrar_servidor($hostname) {
    $this->db->where('hostname = ?',array($hostname));
    return $this->db->delete('Servidors');
  } 
  
  
  /***********************************
   * Persistència grups de servidors *
   ***********************************/
 
  /*
   * Funció que dóna d'alta un grup de servidors
   */
  function insertar_grup($name, $info, $individual) {
    return $this->db->insert('GrupServidors',array('name'=>$name,'info'=>$info,'individual'=>$individual));
  }

  /*
   * Funció que retorna tots els grups de servidors, ordenats per $name de grup
   */
  function consultar_grups() {
    return $this->db->query_all('select * from GrupServidors order by name');
  }  
  
  /*
   * Funció que s'utilitza per obtenir el conjunt de grups de servidor que compleixen
   * la condició que el camp $camp contingui el valor $valor.
   */
  function consultar_grups_filtre($camp,$valor) {
  	
  	switch ($camp) {
  		case "name":
  			return $this->db->query_all('select * from GrupServidors where name like ? order by name',array('%'.$valor."%"));
  			break;
  		case "info":
  			return $this->db->query_all('select * from GrupServidors where info like ? order by name',array('%'.$valor."%"));
  			break;
  	}
  }
  
  /*
   * Funció que retorna un grup de servidors en funció del seu $name 
   */
  function consultar_grup($name) {
    return $this->db->query_all('select * from GrupServidors where name=?',array($name));
  } 
  
  /*
   * Funció que modifica la descripció d'un grup de servidors
   */
  function modificar_grup($name, $info) {
  	$this->db->where('name',$name);
  	return $this->db->update('GrupServidors',array('info'=>$info));
  } 
  
  /*
   * Funció que dóna de baixa un grup de servidors de l'aplicació
   */
  function esborrar_grup($name) {
    $this->db->where('name = ?',array($name)); 
    return $this->db->delete('GrupServidors');
  } 
  
  /* 
   * Funció que dóna d'alta una associació entre un servidor $hostname 
   * i un grup de servidors $groupname
   */
  function insertar_agrupacio($hostname,$groupname) {
	return $this->db->insert('AgrupacioServidors',array('hostname'=>$hostname,'groupname'=>$groupname));
  } 
  
  /*
   * Funció que esborra l'agrupació existent entre un servidor $hostname
   * i un grup de servidors $groupname
   */
  function esborrar_agrupacio($hostname,$groupname) {
  	$this->db->where('hostname = ? and groupname = ?',array($hostname,$groupname)); 
    return $this->db->delete('AgrupacioServidors');
  } 
  
  /*
   * Funció que esborra totes les agrupacions existents d'un servidor $hostname,
   * excepte l'existent entre un $hostname i el seu grup individual
   */
  function esborrar_agrupacions($hostname) {
  	$this->db->where('hostname = ? and groupname != ?',array($hostname,$hostname)); 
    return $this->db->delete('AgrupacioServidors');
  } 
  
  /*
   * Funció que esborra totes les agrupacions existents d'un grup de servidors
   * $groupname
   */
  function esborrar_agrupacions_grup($groupname) {
  	$this->db->where('groupname = ?',array($groupname)); 
    return $this->db->delete('AgrupacioServidors');
  } 
  
  /*
   * Funció que indica si existeix una agrupació entre un servidor $hostname
   * i un grup de servidors $groupname
   */
  function isChecked_grup($hostname,$groupname){
  	return $this->db->query_all('select count(*) as numChecked from AgrupacioServidors where hostname=? and groupname=?',array($hostname,$groupname));
  }
}
?>