<?php
class Usuaris_Model extends TinyMVC_Model
{

  /****************************************************
   * Persistència usuaris (subsistema d'identificació *
   ****************************************************/

  /*
   * Funció que valida si per l'usuari $username, la contrasenya $password
   * és o no correcta: si ho és retorna l'usuari, en cas contrari no.
   */
  function validar_usuari($username,$password){
  	return $this->db->query_all('select count(*) as validacio from Usuaris where username=? and password=?',array($username,$password));
  }
  
  /*
   * Funció que permet canviar la contrasenya de l'usuari $username.
   * El paràmetre $password ja ve codificat amb MD5 des de l'aplicació.
   */
  function canviar_password($username,$password){
  	$this->db->where('username',$username);
  	return $this->db->update('Usuaris',array('password'=>$password));
  }
  
  /************************
   * Persistència usuaris *
   ************************/
 
  /*
   * Funció que retorna l'usuari que té com a nom d'usuari $username
   */
  function consultar_usuari($username) {
    return $this->db->query_all('select * from Usuaris where username=?',array($username));
  } 
   
  /*
   * Funció que retorna tots els usuaris donats d'alta a l'aplicació, ordenats
   * per $username
   */
  function consultar_usuaris() {
    return $this->db->query_all('select * from Usuaris order by username');
  }  
  
  /*
   * Funció que s'utilitza per obtenir el conjunt d'usuaris que compleixen
   * la condició que el camp $camp contingui el valor $valor.
   */
  function consultar_usuaris_filtre($camp,$valor) {
  	
  	switch ($camp) {
  		case "username":
  			return $this->db->query_all('select * from Usuaris where username like ? order by username',array('%'.$valor."%"));
  			break;
  		case "name":
  			return $this->db->query_all('select * from Usuaris where name like ? order by username',array('%'.$valor."%"));
  			break;
  		case "surnames":
  			return $this->db->query_all('select * from Usuaris where surnames like ? order by username',array('%'.$valor."%"));
  			break;
  		case "info":
  			return $this->db->query_all('select * from Usuaris where info like ? order by username',array('%'.$valor."%"));
  			break;  			
  	}   
  }  
  
  /*
   * Funció que dóna d'alta un nou usuari
   */
  function insertar_usuari($username, $name, $surnames, $admin, $password, $info, $pk) {
    return $this->db->insert('Usuaris',array('username'=>$username,'name'=>$name,'surnames'=>$surnames,'admin'=>$admin,'password'=>$password,'info'=>$info,'public_key'=>$pk));
  } 
  
  /*
   * Funció que permet modificar els atributs de l'usuari $username
   */
  function modificar_usuari($username, $name, $surnames, $admin, $info, $pk) {
  	$this->db->where('username',$username);
  	return $this->db->update('Usuaris',array('name'=>$name,'surnames'=>$surnames,'admin'=>$admin,'info'=>$info,'public_key'=>$pk));
  } 
  
  /*
   * Funció que dóna de baixa l'usuari amb nom d'usuari $username
   */
  function esborrar_usuari($username) {
    $this->db->where('username = ?',array($username));
    return $this->db->delete('Usuaris');
  } 
  
  /*
   * Funció que indica si l'usuari $username és o no administrador
   */
  function isAdmin($username) {
  	return $this->db->query_all('select admin from Usuaris where username = ?',array($username));
  }
  
  
  /********************************
   * Persistència grups d'usuaris *
   ********************************/
 
  /*
   * Funció que dóna d'alta un grup d'usuaris a l'aplicació
   */
  function insertar_grup($name, $info, $admin, $individual) {
    return $this->db->insert('GrupUsuaris',array('name'=>$name,'info'=>$info,'admin'=>$admin,'individual'=>$individual));
  }

  /*
   * Funció que retorna tots els grups d'usuaris
   */
  function consultar_grups() {
    return $this->db->query_all('select * from GrupUsuaris order by name');
  }  
  
  /*
   * Funció que s'utilitza per obtenir el conjunt de grups d'usuaris que compleixen
   * la condició que el camp $camp contingui el valor $valor.
   */
  function consultar_grups_filtre($camp,$valor) {
  	
  	switch ($camp) {
  		case "name":
  			return $this->db->query_all('select * from GrupUsuaris where name like ? order by name',array('%'.$valor."%"));
  			break;
  		case "info":
  			return $this->db->query_all('select * from GrupUsuaris where info like ? order by name',array('%'.$valor."%"));
  			break;
  	}
  }
  
  /*
   * Funció que retorna el grup d'usuaris amb nom $name
   */
  function consultar_grup($name) {
    return $this->db->query_all('select * from GrupUsuaris where name=?',array($name));
  } 
  
  /*
   * Funció que permet modificar la descripció $info del grup d'usuaris
   * de nom $name
   */
  function modificar_grup($name, $info) {
  	$this->db->where('name',$name);
  	return $this->db->update('GrupUsuaris',array('info'=>$info));
  } 
  
  /*
   * Funció que dóna de baixa el grup d'usuaris de nom $name
   */
  function esborrar_grup($name) {
    $this->db->where('name = ?',array($name));
    return $this->db->delete('GrupUsuaris');
  } 
  
  /*
   * Funció que dóna d'alta una agrupació entre l'usuari $username i el 
   * grup d'usuaris $groupname 
   */
  function insertar_agrupacio($username,$groupname) {
	return $this->db->insert('AgrupacioUsuaris',array('username'=>$username,'groupname'=>$groupname));
  } 
  
  /*
   * Funció que esborra l'agrupació existent entre l'usuari $username i el
   * grup d'usuaris $groupname
   */
  function esborrar_agrupacio($username,$groupname) {
  	$this->db->where('username = ? and groupname = ?',array($username,$groupname)); 
    return $this->db->delete('AgrupacioUsuaris');
  } 
  
  /*
   * Funció que esborra totes les agrupacions de l'usuari $username,
   * excepte l'existent amb el seu grup individual
   */
  function esborrar_agrupacions($username) {
  	$this->db->where('username = ? and groupname != ?',array($username,$username)); 
    return $this->db->delete('AgrupacioUsuaris');
  } 
  
  /* 
   * Funció que esborra totes les agrupacions existens del grup d'usuaris
   * $groupname
   */
  function esborrar_agrupacions_grup($groupname) {
  	$this->db->where('groupname = ?',array($groupname)); 
    return $this->db->delete('AgrupacioUsuaris');
  } 
  
  /*
   * Funció que indica si existeix o no una agrupació entre l'usuari $username
   * i el grup d'usuaris $groupname
   */
  function isChecked_grup($username,$groupname){
  	return $this->db->query_all('select count(*) as numChecked from AgrupacioUsuaris where username=? and groupname=?',array($username,$groupname));
  }
}
?>